// Jean Luc Biellmann - contact@alsatux.com

"use strict";

var _Storage = {
	debug: false,
	allowed: function () {
		return navigator.getDeviceStorage!==undefined;
	},
	read: function (device, filename, callback_onsuccess, callback_onerror) {
		if (!_Storage.allowed()) {
			_Log.warn('Storage API not available. Unable to read filename "' + device + ':' + filename + '".');
			return;
		}
		var storage = navigator.getDeviceStorage(device);
		if (storage==null) {
			_Log.warn('Storage device "' + device + '" not found.  Unable to read filename "' + filename + '".');
			return;
		}
		var req = storage.get(filename);
		req.onsuccess = function () {
			callback_onsuccess(this);
		}
		req.onerror = function () {
			callback_onerror(this);
		}
	},
	write: function (device, filename, blobdata, blobtype) {
		if (!_Storage.allowed()) {
			_Log.warn('Storage API not available. Unable to write filename "' + device + ':' + filename + '".');
			return;
		}
		var blob = new Blob([blobdata], {type: blobtype});
		var storage = navigator.getDeviceStorage(device);
		if (storage==null) {
			_Log.warn('Storage device "' + device + '" not found.  Unable to write filename "' + filename + '".');
			return;
		}
		var req = storage.addNamed(blob, filename);
		req.onsuccess = function () {
			_Log.info('File "' + device + ':' + filename + '" has been written !');
		}
		req.onerror = function () {
			var extra = (device=='sdcard' ? ' USB not unplugged ?' : '');
			_Log.error('Unable to write "' + device + ':' +  filename + '"' + extra);
		}
	},
	ls: function (device, dir, callback) {
		var files = [];
		if (!_Storage.allowed()) {
			_Log.warn('Storage API not found. List directory "' + dir + '" failed.');
			return callback(files);
		}
		var storage = navigator.getDeviceStorage(device);
		if (storage==null) {
			_Log.warn('Storage device "' + device + '" not found.  Unable to list directory "' + dir + '".');
			return callback(files);
		}
		_Log.info('Listing directory "' + device + ':' + dir + '"...');
		var cursor = storage.enumerate(dir);
		cursor.onsuccess = function () {
			var file = cursor.result;
			if (file != null) {
				if (Storage.debug)
					_Log.info('Add file "' + file.name + '"...');
				files.push(file);
				if (Storage.debug)
					_Log.info('Continue...');
				cursor.continue();
			} else {
				if (Storage.debug)
					_Log.info('Finished...');
				callback(files);
			}
		}
		cursor.onerror = function () {
			var extra = (device=='sdcard' ? ' USB not unplugged ?' : '');
			_Log.warn('Listing directory "' + device + ':' + dir + '" failed.' + extra);
			callback(files);
		}
	}
};

