// Jean Luc Biellmann - contact@alsatux.com

"use strict";

var _XML = {
	debug: false,
	read: function (text) {
		var xml_doc;
		try {
			var str = text.replace('<?xml version="1.0" encoding="UTF-8"?>','');
			var parser = new DOMParser();
			xml_doc = parser.parseFromString(str, 'text/xml');
		} catch (err) {
			_Log.error('Parsing XML failed : ' + err.message);
		}
		// read settings
		try {
			if (xml_doc.getElementsByTagName("settings").length) {
				var xml_settings = xml_doc.getElementsByTagName("settings")[0];
				if (xml_settings.getElementsByTagName("wkst").length) {
					var value = xml_settings.getElementsByTagName('wkst')[0].childNodes[0].nodeValue;
					if (value.match(/(Monday|Sunday)/)!=null)
						_CalGUI.settings['wkst'] = value;
				}
			}
		} catch (err) {
			_Log.error('Reading settings failed: ' + err.message);
		}
		// read events
		var value;
		var xml_events = xml_doc.getElementsByTagName("event");
		//_Log.info('nbevents=' + xml_events.length);
		for (var i=0;i<xml_events.length;i++) {
			var t = [];
			var d = [];
			var xml_event = xml_events[i];
			try {
				// date
				var date = xml_event.getElementsByTagName('date')[0].childNodes[0].nodeValue;
				// time
				value = xml_event.getElementsByTagName('time')[0].childNodes[0].nodeValue;
				if (value.match(/^\d\d:\d\d$/)==null)
					value = "00:00";
				t = value.split(':');
				//duration
				value = xml_event.getElementsByTagName('duration')[0].childNodes[0].nodeValue;
				if (value.match(/^\d\d:\d\d$/)==null)
					value = "01:00";
				d = value.split(':');
				// title
				var title = '';
				if (xml_event.getElementsByTagName('title').length) {
					if (xml_event.getElementsByTagName('title')[0].childNodes.length)
						title = xml_event.getElementsByTagName('title')[0].childNodes[0].nodeValue;
				}
				// description
				var description ='';
				if (xml_event.getElementsByTagName('description').length) {
					if (xml_event.getElementsByTagName('description')[0].childNodes.length)
						description = xml_event.getElementsByTagName('description')[0].childNodes[0].nodeValue;
				}
				if (_XML.debug)
					_Log.info('Creating new event...');
				var new_event = _EventDB.add();
				var obj_event = {
					'seq': new_event.seq,
					'start_date': date,
					'start_hour': t[0],
					'start_minute': t[1],
					'duration_hour': d[0],
					'duration_minute': d[1],
					'title': title,
					'description': description
				};
				// add repeat rule
				var rrule = {
					'freq': '',
					'count': 0,
					'interval': 1,
					'until': ''
				};
				if (xml_event.getElementsByTagName('rrule').length) {
					var xml_rrule = xml_event.getElementsByTagName('rrule')[0];
					if (xml_rrule.getElementsByTagName('freq').length) {
						var freq = null;
						// freq (mandatory)
						try {
							freq = xml_rrule.getElementsByTagName('freq')[0].childNodes[0].nodeValue;
						} catch (err) {
							_Log.error('Bad frequency format...');
						}
						if (freq!=null) {
							if (freq.match(/(yearly|monthly|weekly|daily)/)==null) {
								_Log.error('Bad frequency value...');
							} else {
								// interval (mandatory)
								var interval = 1;
								try {
									interval = _Fmt.modulo(xml_rrule.getElementsByTagName('interval')[0].childNodes[0].nodeValue, 99);
								} catch (err) {
									_Log.error('Bad repeat interval ignored...');
								}
								if (interval<1) {
									_Log.error('Bad interval value...');
								} else {
									rrule['freq'] = freq;
									rrule['interval'] = interval;
									// optional count
									if (xml_rrule.getElementsByTagName('count').length) {
										try {
											rrule['count'] = _Fmt.modulo(xml_rrule.getElementsByTagName('count')[0].childNodes[0].nodeValue, 99);
										} catch (err) {
											_Log.error('Bad repeat count ignored...');
										}
									}
								}
							}
						}
					}
					// optional until
					if (xml_rrule.getElementsByTagName('until').length) {
						var until = null;
						try {
							until = xml_rrule.getElementsByTagName('until')[0].childNodes[0].nodeValue;
						} catch (err) {
							_Log.error('Bad repeat until format...');
						}
						if (until!=null) {
							if (_Check.is_valid_date(until)) {
								rrule['until'] = until;
							} else {
								_Log.error('Bad repeat until value...');
							}
						}
					}
				}
				obj_event['rrule'] = rrule;
				if (_XML.debug) {
					for (var key in obj_event) {
						if (key=='rrule') {
							for (var key2 in obj_event['rrule'])
								_Log.info('   ' + key2 + ': ' + obj_event['rrule'][key2]);
						} else {
							_Log.info(key + ': ' + obj_event[key]);
						}
					}
				}
				_EventDB.upd(obj_event);
			} catch (err) {
				_Log.error('Reading event failed: '+ err.message);
			}
		}
		_CalGUI.refresh_events();
	},
	write: function (filename) {
		var event, markup, xml_doc, xml_settings, xml_events, xml_element, xml_textnode, xml_node, xml_rrule;
		markup = '<?xml version="1.0" encoding="UTF-8"?><calendar name="default"><settings></settings><events></events></calendar>';
		xml_doc = (new DOMParser()).parseFromString(markup, 'application/xml');
		// record settings
		try {
			xml_settings = xml_doc.getElementsByTagName("settings")[0];
			for (var key in _CalGUI.settings) {
				xml_element = xml_doc.createElement(key);
				xml_settings.appendChild(xml_element);
				xml_textnode = xml_doc.createTextNode(_CalGUI.settings[key]);
				xml_element.appendChild(xml_textnode);
			}
		} catch (err) {
			_Log.error('Recording settings failed: ' + err.message);
		}
		// record events
		xml_events = xml_doc.getElementsByTagName("events")[0];
		for (var seq in _EventDB.db) {
			event = _EventDB.db[seq];
			try {
				xml_node = xml_doc.createElement('event');
				xml_events.appendChild(xml_node);
				// date
				xml_element = xml_doc.createElement('date');
				xml_node.appendChild(xml_element);
				xml_textnode = xml_doc.createTextNode(event.start_date);
				xml_element.appendChild(xml_textnode);
				// time
				xml_element = xml_doc.createElement('time');
				xml_node.appendChild(xml_element);
				xml_textnode = xml_doc.createTextNode(_Fmt.two_digits(event.start_hour) + ':' + _Fmt.two_digits(event.start_minute));
				xml_element.appendChild(xml_textnode);
				// duration
				xml_element = xml_doc.createElement('duration');
				xml_node.appendChild(xml_element);
				xml_textnode = xml_doc.createTextNode(_Fmt.two_digits(event.duration_hour) + ':' + _Fmt.two_digits(event.duration_minute));
				xml_element.appendChild(xml_textnode);
				// title
				if (event.title.length) {
					xml_element = xml_doc.createElement('title');
					xml_node.appendChild(xml_element);
					xml_textnode = xml_doc.createTextNode(event.title);
					xml_element.appendChild(xml_textnode);
				}
				// description
				if (event.description.length) {
					xml_element = xml_doc.createElement('description');
					xml_node.appendChild(xml_element);
					xml_textnode = xml_doc.createTextNode(event.description);
					xml_element.appendChild(xml_textnode);
				}
				// rrule
				if (Object.keys(event.rrule).length) {
					xml_rrule = xml_doc.createElement('rrule');
					if ('freq' in event.rrule && event.rrule['freq'].length) {
						xml_element = xml_doc.createElement('freq');
						xml_rrule.appendChild(xml_element);
						xml_textnode = xml_doc.createTextNode(event.rrule['freq']);
						xml_element.appendChild(xml_textnode);
						xml_element = xml_doc.createElement('count');
						xml_rrule.appendChild(xml_element);
						xml_textnode = xml_doc.createTextNode(event.rrule['count']);
						xml_element.appendChild(xml_textnode);
						xml_element = xml_doc.createElement('interval');
						xml_rrule.appendChild(xml_element);
						xml_textnode = xml_doc.createTextNode(event.rrule['interval']);
						xml_element.appendChild(xml_textnode);
					}
					if ('until' in event.rrule && event.rrule['until'].length) {
						xml_element = xml_doc.createElement('until');
						xml_rrule.appendChild(xml_element);
						xml_textnode = xml_doc.createTextNode(event.rrule['until']);
						xml_element.appendChild(xml_textnode);
					}
					if (xml_rrule.childNodes.length)
						xml_node.appendChild(xml_rrule);
				}
			} catch (err) {
				_Log.error('Recording event failed: ' + err.message);
			}
		}
		var blobdata = (new XMLSerializer()).serializeToString(xml_doc);
		if (_Storage.allowed())
			_Storage.write('sdcard', filename, blobdata, 'text/plain');
	}
};

